@@ -198,6 +198,8 @@ int main(int argc,char **argv) {
   output.setProjection(input.getProjection());
   double gt[6];
   input.getGeoTransform(gt);
+  gt[1]*=down_opt[0];//dx
+  gt[5]*=down_opt[0];//dy
   output.setGeoTransform(gt);
   
   if(colorTable_opt.size()){
@@ -560,7 +562,7 @@ int main(int argc,char **argv) {
       theTaps[2][1]=0.0;
       theTaps[2][2]=1.0;
       filter2d.setTaps(theTaps);
-      filter2d.filter(input,output,true);//absolute and normalize
+      filter2d.filter(input,output,true,true);//absolute and normalize
       break;
     }
     case(filter2d::sobely):{//Sobel edge detection in Y
@@ -579,7 +581,7 @@ int main(int argc,char **argv) {
       theTaps[2][1]=-2.0;
       theTaps[2][2]=-1.0;
       filter2d.setTaps(theTaps);
-      filter2d.filter(input,output,true);//absolute and normalize
+      filter2d.filter(input,output,true,true);//absolute and normalize
       break;
     }
     case(filter2d::sobelxy):{//Sobel edge detection in XY
@@ -598,7 +600,7 @@ int main(int argc,char **argv) {
       theTaps[2][1]=-1.0;
       theTaps[2][2]=0.0;
       filter2d.setTaps(theTaps);
-      filter2d.filter(input,output,true);//absolute and normalize
+      filter2d.filter(input,output,true,true);//absolute and normalize
       break;
     }
     case(filter2d::sobelyx):{//Sobel edge detection in XY
@@ -617,7 +619,7 @@ int main(int argc,char **argv) {
       theTaps[2][1]=-1.0;
       theTaps[2][2]=-2.0;
       filter2d.setTaps(theTaps);
-      filter2d.filter(input,output,true);//absolute and normalize
+      filter2d.filter(input,output,true,true);//absolute and normalize
       break;
     }
     case(filter2d::smooth):{//Smoothing filter
@@ -683,8 +685,11 @@ int main(int argc,char **argv) {
 	filter2d.dwtCut(input, output, wavelet_type_opt[0], family_opt[0], threshold_opt[0]);
       break;
     case(filter2d::threshold):
-      filter2d.setThresholds(threshold_opt);
+      filter2d.setThresholds(threshold_opt);//deliberate fall through
+    case(filter2d::density):
       filter2d.setClasses(class_opt);//deliberate fall through
+      if(verbose_opt[0])
+	std::cout << "classes set" << std::endl;
     default:
       filter2d.doit(input,output,method_opt[0],dimX_opt[0],dimY_opt[0],down_opt[0],disc_opt[0]);
       break;
