@@ -42,7 +42,7 @@ int main(int argc, char *argv[])
   Optionpk<bool>  minmax_opt("mm", "minmax", "Shows min and max value of the image ", false,0);
   Optionpk<bool>  min_opt("min", "min", "Shows min value of the image ", false,0);
   Optionpk<bool>  max_opt("max", "max", "Shows max value of the image ", false,0);
-  Optionpk<bool>  stat_opt("stat", "stat", "Shows statistics (min,max, mean and stdDev of the image)", false,0);
+  Optionpk<bool>  stat_opt("stats", "stats", "Shows statistics (min,max, mean and stdDev of the image)", false,0);
   Optionpk<double>  src_min_opt("src_min", "src_min", "Sets minimum for histogram (does not calculate min value: use -mm instead)");
   Optionpk<double>  src_max_opt("src_max", "src_max", "Sets maximum for histogram (does not calculate min value: use -mm instead)");
   Optionpk<bool>  relative_opt("rel", "rel", "Calculates relative histogram in percentage", false,0);
@@ -305,12 +305,16 @@ int main(int argc, char *argv[])
       unsigned long int nsample=imgReader.getHistogram(output,minValue,maxValue,nbin,band_opt[0]);
       std::cout.precision(10);
       for(int bin=0;bin<nbin;++bin){
-	// nsample+=output[bin];
-	std::cout << minValue+static_cast<double>(maxValue-minValue)*(bin+0.5)/nbin << " ";
+	double binValue=0;
+	if(nbin==maxValue-minValue+1)
+	  binValue=minValue+bin;
+	else
+	  binValue=minValue+static_cast<double>(maxValue-minValue)*(bin+0.5)/nbin;
+	std::cout << binValue << " ";
 	if(relative_opt[0])
 	  std::cout << 100.0*static_cast<double>(output[bin])/static_cast<double>(nsample) << std::endl;
 	else
-	  std::cout << static_cast<double>(output[bin])  << std::endl;
+	  std::cout << static_cast<double>(output[bin]) << std::endl;
       }
     }
     // else{
