@@ -26,6 +26,8 @@ along with pktools.  If not, see <http://www.gnu.org/licenses/>.
 #include "algorithms/StatFactory.h"
 #include "algorithms/Filter2d.h"
 
+using namespace std;
+
 int main(int argc,char **argv) {
   Optionpk<string> input_opt("i", "input", "Input las file");
   Optionpk<short> nodata_opt("nodata", "nodata", "nodata value to put in image if not valid", 0);
@@ -33,21 +35,21 @@ int main(int argc,char **argv) {
   Optionpk<bool> disc_opt("circ", "circular", "circular disc kernel for dilation and erosion", false);
   Optionpk<double> maxSlope_opt("s", "maxSlope", "Maximum slope used for morphological filtering", 0.0);
   Optionpk<double> hThreshold_opt("ht", "maxHeight", "initial and maximum height threshold for progressive morphological filtering (e.g., -ht 0.2 -ht 2.5)", 0.2);
-  Optionpk<short> maxIter_opt("\0", "maxIter", "Maximum number of iterations in post filter", 100.0);
-  Optionpk<short> nbin_opt("nb", "nbin", "Number of percentile bins for calculating profile (=number of output bands)", 10.0);
-  Optionpk<unsigned short> returns_opt("r", "returns", "number(s) of returns to include");
-  Optionpk<unsigned short> classes_opt("c", "classes", "classes to keep: 0 (created, never classified), 1 (unclassified), 2 (ground), 3 (low vegetation), 4 (medium vegetation), 5 (high vegetation), 6 (building), 7 (low point, noise), 8 (model key-point), 9 (water), 10 (reserved), 11 (reserved), 12 (overlap)");
+  Optionpk<short> maxIter_opt("maxit", "maxit", "Maximum number of iterations in post filter", 5);
+  Optionpk<short> nbin_opt("nbin", "nbin", "Number of percentile bins for calculating profile (=number of output bands)", 10.0);
+  Optionpk<unsigned short> returns_opt("ret", "ret", "number(s) of returns to include");
+  Optionpk<unsigned short> classes_opt("class", "class", "classes to keep: 0 (created, never classified), 1 (unclassified), 2 (ground), 3 (low vegetation), 4 (medium vegetation), 5 (high vegetation), 6 (building), 7 (low point, noise), 8 (model key-point), 9 (water), 10 (reserved), 11 (reserved), 12 (overlap)");
   Optionpk<string> composite_opt("comp", "comp", "composite for multiple points in cell (min, max, median, mean, sum, first, last, profile, number (point density)). Last: overwrite cells with latest point", "last");
   Optionpk<string> filter_opt("fir", "filter", "filter las points (last,single,multiple,all).", "all");
-  Optionpk<string> postFilter_opt("pf", "pfilter", "post processing filter (etew_min,promorph (progressive morphological filter),bunting (adapted promorph),open,close,none).", "none");
-  Optionpk<short> dimx_opt("\0", "dimX", "Dimension X of postFilter", 3);
-  Optionpk<short> dimy_opt("\0", "dimY", "Dimension Y of postFilter", 3);
+  // Optionpk<string> postFilter_opt("pf", "pfilter", "post processing filter (etew_min,promorph (progressive morphological filter),bunting (adapted promorph),open,close,none).", "none");
+  // Optionpk<short> dimx_opt("dimx", "dimx", "Dimension X of postFilter", 3);
+  // Optionpk<short> dimy_opt("dimy", "dimy", "Dimension Y of postFilter", 3);
   Optionpk<string> output_opt("o", "output", "Output image file");
   Optionpk<string> projection_opt("a_srs", "a_srs", "assign the projection for the output file in epsg code, e.g., epsg:3035 for European LAEA projection");
-  Optionpk<double> ulx_opt("\0", "ulx", "Upper left x value bounding box (in geocoordinates if georef is true). 0 is read from input file", 0.0);
-  Optionpk<double> uly_opt("\0", "uly", "Upper left y value bounding box (in geocoordinates if georef is true). 0 is read from input file", 0.0);
-  Optionpk<double> lrx_opt("\0", "lrx", "Lower right x value bounding box (in geocoordinates if georef is true). 0 is read from input file", 0.0);
-  Optionpk<double> lry_opt("\0", "lry", "Lower right y value bounding box (in geocoordinates if georef is true). 0 is read from input file", 0.0);
+  Optionpk<double> ulx_opt("ulx", "ulx", "Upper left x value bounding box (in geocoordinates if georef is true). 0 is read from input file", 0.0);
+  Optionpk<double> uly_opt("uly", "uly", "Upper left y value bounding box (in geocoordinates if georef is true). 0 is read from input file", 0.0);
+  Optionpk<double> lrx_opt("lrx", "lrx", "Lower right x value bounding box (in geocoordinates if georef is true). 0 is read from input file", 0.0);
+  Optionpk<double> lry_opt("lry", "lry", "Lower right y value bounding box (in geocoordinates if georef is true). 0 is read from input file", 0.0);
   Optionpk<string> otype_opt("ot", "otype", "Data type for output image ({Byte/Int16/UInt16/UInt32/Int32/Float32/Float64/CInt16/CInt32/CFloat32/CFloat64}). Empty string: inherit type from input image", "Byte");
   Optionpk<string> oformat_opt("of", "oformat", "Output image format (see also gdal_translate). Empty string: inherit from input image", "GTiff");
   Optionpk<string> option_opt("co", "co", "Creation option for output file. Multiple options can be specified.");
@@ -72,9 +74,9 @@ int main(int argc,char **argv) {
     classes_opt.retrieveOption(argc,argv);
     composite_opt.retrieveOption(argc,argv);
     filter_opt.retrieveOption(argc,argv);
-    postFilter_opt.retrieveOption(argc,argv);
-    dimx_opt.retrieveOption(argc,argv);
-    dimy_opt.retrieveOption(argc,argv);
+    // postFilter_opt.retrieveOption(argc,argv);
+    // dimx_opt.retrieveOption(argc,argv);
+    // dimy_opt.retrieveOption(argc,argv);
     output_opt.retrieveOption(argc,argv);
     projection_opt.retrieveOption(argc,argv);
     ulx_opt.retrieveOption(argc,argv);
@@ -188,13 +190,13 @@ int main(int argc,char **argv) {
     std::cout << setprecision(12) << "--ulx=" << minULX << " --uly=" << maxULY << " --lrx=" << maxLRX << " --lry=" << minLRY << std::endl;
     std::cout << "total number of points before filtering: " << totalPoints << std::endl;
     std::cout << "filter set to " << filter_opt[0] << std::endl;
-    std::cout << "postFilter set to " << postFilter_opt[0] << std::endl;
+    // std::cout << "postFilter set to " << postFilter_opt[0] << std::endl;
   }
   int ncol=ceil(maxLRX-minULX)/dx_opt[0];//number of columns in outputGrid
   int nrow=ceil(maxULY-minLRY)/dy_opt[0];//number of rows in outputGrid
   //todo: multiple bands
   int nband=(composite_opt[0]=="profile")? nbin_opt[0] : 1;
-  if(output_opt.size()){
+  if(!output_opt.size()){
     cerr << "Error: no output file defined" << endl;
     exit(1);
   }
@@ -203,7 +205,14 @@ int main(int argc,char **argv) {
   outputWriter.open(output_opt[0],ncol,nrow,nband,theType,oformat_opt[0],option_opt);
   outputWriter.GDALSetNoDataValue(nodata_opt[0]);
   //set projection
-  outputWriter.setGeoTransform(minULX,maxULY,dx_opt[0],dy_opt[0],0,0);
+  double gt[6];
+  gt[0]=minULX;
+  gt[1]=dx_opt[0];
+  gt[2]=0;
+  gt[3]=maxULY;
+  gt[4]=0;
+  gt[5]=-dy_opt[0];
+  outputWriter.setGeoTransform(gt);
   if(projection_opt.size()){
     string projectionString=outputWriter.setProjectionProj4(projection_opt[0]);
     if(verbose_opt[0])
@@ -285,12 +294,16 @@ int main(int argc,char **argv) {
       outputWriter.geo2image(thePoint.GetX(),thePoint.GetY(),dcol,drow);
       int icol=static_cast<int>(dcol);
       int irow=static_cast<int>(drow);
-      // //test
-      // irow+=1;
-      if(irow<0||irow>=nrow)
+      if(irow<0||irow>=nrow){
+	// //test
+	// cout << "Error: thePoint.GetX(),thePoint.GetY(),dcol,drow" << thePoint.GetX() << ", " << thePoint.GetY() << ", " << dcol << ", " << drow << endl;
 	continue;
-      if(icol<0||icol>=ncol)
+      }
+      if(icol<0||icol>=ncol){
+	// //test
+	// cout << "Error: thePoint.GetX(),thePoint.GetY(),dcol,drow" << thePoint.GetX() << ", " << thePoint.GetY() << ", " << dcol << ", " << drow << endl;
 	continue;
+      }
       assert(irow>=0);
       assert(irow<nrow);
       assert(icol>=0);
@@ -323,11 +336,11 @@ int main(int argc,char **argv) {
   pfnProgress(progress,pszMessage,pProgressArg);
   statfactory::StatFactory stat;
   //fill inputData in outputData
-  if(composite_opt[0]=="profile"){
-    assert(postFilter_opt[0]=="none");
+  // if(composite_opt[0]=="profile"){
+    // assert(postFilter_opt[0]=="none");
     // for(int iband=0;iband<nband;++iband)
       // outputProfile[iband].resize(nrow,ncol);
-  }
+  // }
   for(int irow=0;irow<nrow;++irow){
     if(composite_opt[0]=="number")
       continue;//outputData already set
@@ -400,102 +413,102 @@ int main(int argc,char **argv) {
   pfnProgress(progress,pszMessage,pProgressArg);
   inputData.clear();//clean up memory
   //apply post filter
-  std::cout << "Applying post processing filter: " << postFilter_opt[0] << std::endl;
-  if(postFilter_opt[0]=="etew_min"){
-    if(composite_opt[0]!="min")
-      std::cout << "Warning: composite option is not set to min!" << std::endl;
-    //Elevation Threshold with Expand Window (ETEW) Filter (p.73 frmo Airborne LIDAR Data Processing and Analysis Tools ALDPAT 1.0)
-    //first iteration is performed assuming only minima are selected using options -fir all -c min
-    unsigned long int nchange=1;
-    //increase cells and thresholds until no points from the previous iteration are discarded.
-    int dimx=dimx_opt[0];
-    int dimy=dimy_opt[0];
-    filter2d::Filter2d morphFilter;
-    morphFilter.setNoValue(0);
-    Vector2d<float> currentOutput=outputData;
-    int iteration=1;
-    while(nchange&&iteration<maxIter_opt[0]){
-      double hThreshold=maxSlope_opt[0]*dimx;
-      Vector2d<float> newOutput;
-      nchange=morphFilter.morphology(currentOutput,newOutput,"erode",dimx,dimy,disc_opt[0],hThreshold);
-      currentOutput=newOutput;
-      dimx+=2;//change from theory: originally double cellCize
-      dimy+=2;//change from theory: originally double cellCize
-      std::cout << "iteration " << iteration << ": " << nchange << " pixels changed" << std::endl;
-      ++iteration;
-    }
-    outputData=currentOutput;
-  }    
-  else if(postFilter_opt[0]=="promorph"||postFilter_opt[0]=="bunting"){
-    if(composite_opt[0]!="min")
-      std::cout << "Warning: composite option is not set to min!" << std::endl;
-    assert(hThreshold_opt.size()>1);
-    //Progressive morphological filter tgrs2003_zhang vol41 pp 872-882
-    //first iteration is performed assuming only minima are selected using options -fir all -c min
-    //increase cells and thresholds until no points from the previous iteration are discarded.
-    int dimx=dimx_opt[0];
-    int dimy=dimy_opt[0];
-    filter2d::Filter2d theFilter;
-    theFilter.setNoValue(0);
-    Vector2d<float> currentOutput=outputData;
-    double hThreshold=hThreshold_opt[0];
-    int iteration=1;
-    while(iteration<maxIter_opt[0]){
-      std::cout << "iteration " << iteration << " with window size " << dimx << " and dh_max: " << hThreshold << std::endl;
-      Vector2d<float> newOutput;
-      try{
-        theFilter.morphology(outputData,currentOutput,"erode",dimx,dimy,disc_opt[0],maxSlope_opt[0]);
-        theFilter.morphology(currentOutput,outputData,"dilate",dimx,dimy,disc_opt[0],maxSlope_opt[0]);
-        if(postFilter_opt[0]=="bunting"){//todo: implement doit in Filter2d on Vector2d
-          theFilter.doit(outputData,currentOutput,"median",dimx,dimy,1,disc_opt[0]);
-          outputData=currentOutput;
-        }
-      }
-      catch(std::string errorString){
-        cout << errorString << endl;
-        exit(1);
-      }
-      int newdimx=(dimx==1)? 3: 2*(dimx-1)+1;
-      int newdimy=(dimx==1)? 3: 2*(dimy-1)+1;//from PE&RS vol 71 pp313-324
-      hThreshold=hThreshold_opt[0]+maxSlope_opt[0]*(newdimx-dimx)*dx_opt[0];
-      dimx=newdimx;
-      dimy=newdimy;
-      if(hThreshold>hThreshold_opt[1])
-        hThreshold=hThreshold_opt[1];
-      ++iteration;
-    }
-    outputData=currentOutput;
-  }    
-  else if(postFilter_opt[0]=="open"){
-    if(composite_opt[0]!="min")
-      std::cout << "Warning: composite option is not set to min!" << std::endl;
-    filter2d::Filter2d morphFilter;
-    morphFilter.setNoValue(0);
-    Vector2d<float> filterInput=outputData;
-    try{
-      morphFilter.morphology(outputData,filterInput,"erode",dimx_opt[0],dimy_opt[0],disc_opt[0],maxSlope_opt[0]);
-      morphFilter.morphology(filterInput,outputData,"dilate",dimx_opt[0],dimy_opt[0],disc_opt[0],maxSlope_opt[0]);
-    }
-    catch(std::string errorString){
-      cout << errorString << endl;
-      exit(1);
-    }
-  }
-  else if(postFilter_opt[0]=="close"){
-    if(composite_opt[0]!="max")
-      std::cout << "Warning: composite option is not set to max!" << std::endl;
-    filter2d::Filter2d morphFilter;
-    morphFilter.setNoValue(0);
-    Vector2d<float> filterInput=outputData;
-    try{
-      morphFilter.morphology(outputData,filterInput,"dilate",dimx_opt[0],dimy_opt[0],disc_opt[0],maxSlope_opt[0]);
-      morphFilter.morphology(filterInput,outputData,"erode",dimx_opt[0],dimy_opt[0],disc_opt[0],maxSlope_opt[0]);
-    }
-    catch(std::string errorString){
-      cout << errorString << endl;
-      exit(1);
-    }
-  }
+  // std::cout << "Applying post processing filter: " << postFilter_opt[0] << std::endl;
+  // if(postFilter_opt[0]=="etew_min"){
+  //   if(composite_opt[0]!="min")
+  //     std::cout << "Warning: composite option is not set to min!" << std::endl;
+  //   //Elevation Threshold with Expand Window (ETEW) Filter (p.73 frmo Airborne LIDAR Data Processing and Analysis Tools ALDPAT 1.0)
+  //   //first iteration is performed assuming only minima are selected using options -fir all -comp min
+  //   unsigned long int nchange=1;
+  //   //increase cells and thresholds until no points from the previous iteration are discarded.
+  //   int dimx=dimx_opt[0];
+  //   int dimy=dimy_opt[0];
+  //   filter2d::Filter2d morphFilter;
+  //   // morphFilter.setNoValue(0);
+  //   Vector2d<float> currentOutput=outputData;
+  //   int iteration=1;
+  //   while(nchange&&iteration<=maxIter_opt[0]){
+  //     double hThreshold=maxSlope_opt[0]*dimx;
+  //     Vector2d<float> newOutput;
+  //     nchange=morphFilter.morphology(currentOutput,newOutput,"erode",dimx,dimy,disc_opt[0],hThreshold);
+  //     currentOutput=newOutput;
+  //     dimx+=2;//change from theory: originally double cellCize
+  //     dimy+=2;//change from theory: originally double cellCize
+  //     std::cout << "iteration " << iteration << ": " << nchange << " pixels changed" << std::endl;
+  //     ++iteration;
+  //   }
+  //   outputData=currentOutput;
+  // }    
+  // else if(postFilter_opt[0]=="promorph"||postFilter_opt[0]=="bunting"){
+  //   if(composite_opt[0]!="min")
+  //     std::cout << "Warning: composite option is not set to min!" << std::endl;
+  //   assert(hThreshold_opt.size()>1);
+  //   //Progressive morphological filter tgrs2003_zhang vol41 pp 872-882
+  //   //first iteration is performed assuming only minima are selected using options -fir all -comp min
+  //   //increase cells and thresholds until no points from the previous iteration are discarded.
+  //   int dimx=dimx_opt[0];
+  //   int dimy=dimy_opt[0];
+  //   filter2d::Filter2d theFilter;
+  //   // theFilter.setNoValue(0);
+  //   Vector2d<float> currentOutput=outputData;
+  //   double hThreshold=hThreshold_opt[0];
+  //   int iteration=1;
+  //   while(iteration<=maxIter_opt[0]){
+  //     std::cout << "iteration " << iteration << " with window size " << dimx << " and dh_max: " << hThreshold << std::endl;
+  //     Vector2d<float> newOutput;
+  //     try{
+  //       theFilter.morphology(outputData,currentOutput,"erode",dimx,dimy,disc_opt[0],hThreshold);
+  //       theFilter.morphology(currentOutput,outputData,"dilate",dimx,dimy,disc_opt[0],hThreshold);
+  //       if(postFilter_opt[0]=="bunting"){
+  //         theFilter.doit(outputData,currentOutput,"median",dimx,dimy,1,disc_opt[0]);
+  //         outputData=currentOutput;
+  //       }
+  //     }
+  //     catch(std::string errorString){
+  //       cout << errorString << endl;
+  //       exit(1);
+  //     }
+  //     int newdimx=(dimx==1)? 3: 2*(dimx-1)+1;
+  //     int newdimy=(dimx==1)? 3: 2*(dimy-1)+1;//from PE&RS vol 71 pp313-324
+  //     hThreshold=hThreshold_opt[0]+maxSlope_opt[0]*(newdimx-dimx)*dx_opt[0];
+  //     dimx=newdimx;
+  //     dimy=newdimy;
+  //     if(hThreshold>hThreshold_opt[1])
+  //       hThreshold=hThreshold_opt[1];
+  //     ++iteration;
+  //   }
+  //   outputData=currentOutput;
+  // }    
+  // else if(postFilter_opt[0]=="open"){
+  //   if(composite_opt[0]!="min")
+  //     std::cout << "Warning: composite option is not set to min!" << std::endl;
+  //   filter2d::Filter2d morphFilter;
+  //   // morphFilter.setNoValue(0);
+  //   Vector2d<float> filterInput=outputData;
+  //   try{
+  //     morphFilter.morphology(outputData,filterInput,"erode",dimx_opt[0],dimy_opt[0],disc_opt[0],maxSlope_opt[0]);
+  //     morphFilter.morphology(filterInput,outputData,"dilate",dimx_opt[0],dimy_opt[0],disc_opt[0],maxSlope_opt[0]);
+  //   }
+  //   catch(std::string errorString){
+  //     cout << errorString << endl;
+  //     exit(1);
+  //   }
+  // }
+  // else if(postFilter_opt[0]=="close"){
+  //   if(composite_opt[0]!="max")
+  //     std::cout << "Warning: composite option is not set to max!" << std::endl;
+  //   filter2d::Filter2d morphFilter;
+  //   // morphFilter.setNoValue(0);
+  //   Vector2d<float> filterInput=outputData;
+  //   try{
+  //     morphFilter.morphology(outputData,filterInput,"dilate",dimx_opt[0],dimy_opt[0],disc_opt[0],maxSlope_opt[0]);
+  //     morphFilter.morphology(filterInput,outputData,"erode",dimx_opt[0],dimy_opt[0],disc_opt[0],maxSlope_opt[0]);
+  //   }
+  //   catch(std::string errorString){
+  //     cout << errorString << endl;
+  //     exit(1);
+  //   }
+  // }
   if(composite_opt[0]!="profile"){
     //write output file
     std::cout << "writing output raster file" << std::endl;
